@@ -1,4 +1,4 @@
-# Copyright (C) 2018-2019 2ndQuadrant Limited
+# Copyright (C) 2018-2020 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -25,7 +25,7 @@ from contextlib import closing
 from io import BytesIO
 
 import barman
-from barman.cloud import CloudInterface
+from barman.cloud import CloudInterface, configure_logging
 from barman.utils import force_str
 from barman.xlog import hash_dir, is_any_xlog_file
 
@@ -34,8 +34,6 @@ try:
 except ImportError:
     raise SystemExit("Missing required python module: argparse")
 
-LOGGING_FORMAT = "%(asctime)s %(levelname)s %(message)s"
-
 
 def main(args=None):
     """
@@ -45,7 +43,7 @@ def main(args=None):
         it defaults to sys.args[1:]
     """
     config = parse_arguments(args)
-    configure_logging()
+    configure_logging(config)
 
     # Validate the WAL file name before uploading it
     if not is_any_xlog_file(config.wal_path):
@@ -54,9 +52,10 @@ def main(args=None):
 
     try:
         cloud_interface = CloudInterface(
-            destination_url=config.destination_url,
+            url=config.destination_url,
             encryption=config.encryption,
-            profile_name=config.profile)
+            profile_name=config.profile,
+            endpoint_url=config.endpoint_url)
 
         with closing(cloud_interface):
             uploader = S3WalUploader(
@@ -91,7 +90,8 @@ def parse_arguments(args=None):
     parser = argparse.ArgumentParser(
         description='This script can be used in the `archive_command` '
                     'of a PostgreSQL server to ship WAL files to the Cloud. '
-                    'Currently only AWS S3 is supported.'
+                    'Currently only AWS S3 is supported.',
+        add_help=False
     )
     parser.add_argument(
         'destination_url',
@@ -111,6 +111,21 @@ def parse_arguments(args=None):
         '-V', '--version',
         action='version', version='%%(prog)s %s' % barman.__version__
     )
+    parser.add_argument(
+        '--help',
+        action='help',
+        help='show this help message and exit')
+    verbosity = parser.add_mutually_exclusive_group()
+    verbosity.add_argument(
+        '-v', '--verbose',
+        action='count',
+        default=0,
+        help='increase output verbosity (e.g., -vv is more than -v)')
+    verbosity.add_argument(
+        '-q', '--quiet',
+        action='count',
+        default=0,
+        help='decrease output verbosity (e.g., -qq is less than -q)')
     parser.add_argument(
         '-P', '--profile',
         help='profile name (e.g. INI section in AWS credentials file)',
@@ -143,16 +158,13 @@ def parse_arguments(args=None):
         action="store_true",
         default=False
     )
+    parser.add_argument(
+        "--endpoint-url",
+        help="Override default S3 endpoint URL with the given one",
+    )
     return parser.parse_args(args=args)
 
 
-def configure_logging():
-    """
-    Get a nicer output from the Python logging package
-    """
-    logging.basicConfig(format=LOGGING_FORMAT, level=logging.ERROR)
-
-
 class S3WalUploader(object):
     """
     S3 upload client
@@ -169,7 +181,6 @@ class S3WalUploader(object):
         """
 
         self.cloud_interface = cloud_interface
-        # If netloc is not present, the s3 url is badly formatted.
         self.compression = compression
         self.server_name = server_name
 
@@ -192,11 +203,6 @@ class S3WalUploader(object):
             wal_name
         )
 
-        # Remove initial "/", otherwise we will create a folder with an empty
-        # name.
-        if destination[0] == '/':
-            destination = destination[1:]
-
         # Put the file in the correct bucket.
         # The put method will handle automatically multipart upload
         self.cloud_interface.upload_fileobj(
