@@ -7,7 +7,7 @@
 #
 # See the help page for usage information.
 #
-# Copyright (C) 2016-2019 2ndQuadrant Limited
+# Copyright (C) 2016-2020 2ndQuadrant Limited
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -182,6 +182,8 @@ def build_ssh_command(config, wal_name, peek=0):
     """
     ssh_command = [
         'ssh',
+        '-q',  # quiet mode - suppress warnings
+        '-T',  # disable pseudo-terminal allocation
         "%s@%s" % (config.user, config.barman_host),
         "barman",
     ]
