@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2018 2ndQuadrant Limited
+# Copyright (C) 2013-2020 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -596,7 +596,6 @@ class LocalBackupInfo(BackupInfo):
                     'both info_file and backup_id parameters are set')
             self.backup_id = backup_id
             self.filename = self.get_filename()
-            self.systemid = server.systemid
             # Check if a backup info file for a given server and a given ID
             # already exists. If so load the values from the file.
             if os.path.exists(self.filename):
@@ -680,7 +679,7 @@ class LocalBackupInfo(BackupInfo):
 
         # Build the requested path according to backup_version value
         path = [self.get_basebackup_directory()]
-        # Check te version of the backup
+        # Check the version of the backup
         if self.backup_version == 2:
             # If an oid has been provided, we are looking for a tablespace
             if tablespace_oid is not None:
