@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2018 2ndQuadrant Limited
+# Copyright (C) 2011-2020 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -29,16 +29,13 @@ from psycopg2.errorcodes import (DUPLICATE_OBJECT, OBJECT_IN_USE,
 from psycopg2.extensions import STATUS_IN_TRANSACTION, STATUS_READY
 from psycopg2.extras import DictCursor, NamedTupleCursor
 
-from barman.exceptions import (ConninfoException, PostgresAppNameError,
-                               PostgresConnectionError,
-                               PostgresDuplicateReplicationSlot,
-                               PostgresException,
-                               PostgresInvalidReplicationSlot,
-                               PostgresIsInRecovery,
-                               PostgresReplicationSlotInUse,
-                               PostgresReplicationSlotsFull,
-                               PostgresSuperuserRequired,
-                               PostgresUnsupportedFeature)
+from barman.exceptions import (
+    ConninfoException, PostgresAppNameError, PostgresConnectionError,
+    PostgresDuplicateReplicationSlot, PostgresException,
+    PostgresInvalidReplicationSlot, PostgresIsInRecovery,
+    PostgresReplicationSlotInUse, PostgresReplicationSlotsFull,
+    BackupFunctionsAccessRequired,
+    PostgresSuperuserRequired, PostgresUnsupportedFeature)
 from barman.infofile import Tablespace
 from barman.postgres_plumbing import function_name_map
 from barman.remote_status import RemoteStatusMixin
@@ -140,6 +137,11 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
         if application_name and 'application_name' not in self.conn_parameters:
             # Then add the it to the connection string
             conn_string += ' application_name=%s' % application_name
+        # adopt a secure schema-usage pattern. See:
+        # https://www.postgresql.org/docs/current/libpq-connect.html
+        if 'options' not in self.conn_parameters:
+            conn_string += ' options=-csearch_path='
+
         return conn_string
 
     def connect(self):
@@ -331,7 +333,7 @@ class StreamingConnection(PostgreSQL):
             # to a replication backend
             if row:
                 result['streaming'] = True
-                # IDENTIFY_SYSTEM always return at least two values
+                # IDENTIFY_SYSTEM always returns at least two values
                 result['streaming_systemid'] = row[0]
                 result['timeline'] = row[1]
                 # PostgreSQL 9.1+ returns also the current xlog flush location
@@ -359,6 +361,8 @@ class StreamingConnection(PostgreSQL):
             # connection, otherwise if will fail with a generic
             # "syntax error"
             cursor.execute('CREATE_REPLICATION_SLOT %s PHYSICAL' % slot_name)
+            _logger.info("Replication slot '%s' successfully created",
+                         slot_name)
         except psycopg2.DatabaseError as exc:
             if exc.pgcode == DUPLICATE_OBJECT:
                 # A replication slot with the same name exists
@@ -383,6 +387,8 @@ class StreamingConnection(PostgreSQL):
             # connection, otherwise if will fail with a generic
             # "syntax error"
             cursor.execute('DROP_REPLICATION_SLOT %s' % slot_name)
+            _logger.info("Replication slot '%s' successfully dropped",
+                         slot_name)
         except psycopg2.DatabaseError as exc:
             if exc.pgcode == UNDEFINED_OBJECT:
                 # A replication slot with the that name does not exist
@@ -508,6 +514,62 @@ class PostgreSQLConnection(PostgreSQL):
                           force_str(e).strip())
             return None
 
+    @property
+    def has_backup_privileges(self):
+        """
+        Returns true if current user is superuser or, for PostgreSQL 10
+        or above, is a standard user that has grants to read server
+        settings and to execute all the functions needed for
+        exclusive/concurrent backup control and WAL control.
+        """
+        # pg_monitor / pg_read_all_settings only available from v10
+        if self.server_version < 100000:
+            return self.is_superuser
+
+        backup_check_query = """
+        SELECT
+          usesuper
+          OR
+          (
+            userepl
+            AND
+            (
+              pg_has_role(CURRENT_USER, 'pg_monitor', 'MEMBER')
+              OR
+              (
+                pg_has_role(CURRENT_USER, 'pg_read_all_settings', 'MEMBER')
+                AND pg_has_role(CURRENT_USER, 'pg_read_all_stats', 'MEMBER')
+              )
+            )
+            AND has_function_privilege(
+              CURRENT_USER, 'pg_start_backup(text,bool,bool)', 'EXECUTE')
+            AND
+            (
+              has_function_privilege(
+                CURRENT_USER, 'pg_stop_backup()', 'EXECUTE')
+              OR has_function_privilege(
+                CURRENT_USER, 'pg_stop_backup(bool,bool)', 'EXECUTE')
+            )
+            AND has_function_privilege(
+              CURRENT_USER, 'pg_switch_wal()', 'EXECUTE')
+            AND has_function_privilege(
+              CURRENT_USER, 'pg_create_restore_point(text)', 'EXECUTE')
+          )
+        FROM
+          pg_user
+        WHERE
+          usename = CURRENT_USER
+        """
+        try:
+            cur = self._cursor()
+            cur.execute(backup_check_query)
+            return cur.fetchone()[0]
+        except (PostgresConnectionError, psycopg2.Error) as e:
+            _logger.debug("Error checking privileges for functions "
+                          "needed for backups: %s",
+                          force_str(e).strip())
+            return None
+
     @property
     def current_xlog_info(self):
         """
@@ -623,9 +685,10 @@ class PostgreSQLConnection(PostgreSQL):
     @property
     def current_size(self):
         """
-        Returns the total size of the PostgreSQL server (requires superuser)
+        Returns the total size of the PostgreSQL server
+        (requires superuser or pg_read_all_stats)
         """
-        if not self.is_superuser:
+        if not self.has_backup_privileges:
             return None
 
         try:
@@ -764,6 +827,7 @@ class PostgreSQLConnection(PostgreSQL):
                 pg_settings.append('wal_level')
                 pg_settings.append('hot_standby')
                 pg_settings.append('max_wal_senders')
+                pg_settings.append('wal_keep_segments')
 
             if self.server_version >= 90300:
                 pg_settings.append('data_checksums')
@@ -775,7 +839,7 @@ class PostgreSQLConnection(PostgreSQL):
                 pg_settings.append('wal_compression')
 
             # retrieves superuser settings
-            if self.is_superuser:
+            if self.has_backup_privileges:
                 for name in pg_superuser_settings:
                     result[name] = self.get_setting(name)
 
@@ -784,6 +848,7 @@ class PostgreSQLConnection(PostgreSQL):
                 result[name] = self.get_setting(name)
 
             result['is_superuser'] = self.is_superuser
+            result['has_backup_privileges'] = self.has_backup_privileges
             result['is_in_recovery'] = self.is_in_recovery
             result['server_txt_version'] = self.server_txt_version
             result['pgespresso_installed'] = self.has_pgespresso
@@ -896,7 +961,7 @@ class PostgreSQLConnection(PostgreSQL):
                 self.configuration_files[cname] = cpath
 
             # Retrieve additional configuration files
-            # If PostgresSQL is older than 8.4 disable this check
+            # If PostgreSQL is older than 8.4 disable this check
             if self.server_version >= 80400:
                 cur.execute(
                     "SELECT DISTINCT sourcefile AS included_file "
@@ -946,6 +1011,8 @@ class PostgreSQLConnection(PostgreSQL):
             cur = self._cursor()
             cur.execute(
                 "SELECT pg_create_restore_point(%s)", [target_name])
+            _logger.info("Restore point '%s' successfully created",
+                         target_name)
             return cur.fetchone()[0]
         except (PostgresConnectionError, psycopg2.Error) as e:
             _logger.debug('Error issuing pg_create_restore_point()'
@@ -1213,8 +1280,8 @@ class PostgreSQLConnection(PostgreSQL):
         try:
             conn = self.connect()
             # Requires superuser privilege
-            if not self.is_superuser:
-                raise PostgresSuperuserRequired()
+            if not self.has_backup_privileges:
+                raise BackupFunctionsAccessRequired()
 
             # If this server is in recovery there is nothing to do
             if self.is_in_recovery:
@@ -1270,10 +1337,8 @@ class PostgreSQLConnection(PostgreSQL):
         try:
             cur = self._cursor(cursor_factory=NamedTupleCursor)
 
-            # Without superuser rights, this function is useless
-            # TODO: provide a simplified version for non-superusers
-            if not self.is_superuser:
-                raise PostgresSuperuserRequired()
+            if not self.has_backup_privileges:
+                raise BackupFunctionsAccessRequired()
 
             # pg_stat_replication is a system view that contains one
             # row per WAL sender process with information about the
