@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2018 2ndQuadrant Limited
+# Copyright (C) 2011-2020 2ndQuadrant Limited
 #
 # This file is part of Barman.
 #
@@ -135,6 +135,21 @@ def parse_log_level(log_level):
     return None
 
 
+# noinspection PyProtectedMember
+def get_log_levels():
+    """
+    Return a list of available log level names
+    """
+    try:
+        level_to_name = logging._levelToName
+    except AttributeError:
+        level_to_name = dict([(key, logging._levelNames[key])
+                              for key in logging._levelNames
+                              if isinstance(key, int)])
+    for level in sorted(level_to_name):
+        yield level_to_name[level]
+
+
 def pretty_size(size, unit=1024):
     """
     This function returns a pretty representation of a size value
@@ -146,7 +161,7 @@ def pretty_size(size, unit=1024):
     suffixes = ["B"] + [i + {1000: "B", 1024: "iB"}[unit] for i in "KMGTPEZY"]
     if unit == 1000:
         suffixes[1] = 'kB'  # special case kB instead of KB
-    # cast to float to avoid loosing decimals
+    # cast to float to avoid losing decimals
     size = float(size)
     for suffix in suffixes:
         if abs(size) < unit or suffix == suffixes[-1]:
@@ -314,7 +329,8 @@ def fsync_dir(dir_path):
         # raises an EINVAL error. Ignoring it is usually safe.
         if e.errno != errno.EINVAL:
             raise
-    os.close(dir_fd)
+    finally:
+        os.close(dir_fd)
 
 
 def fsync_file(file_path):
@@ -329,8 +345,24 @@ def fsync_file(file_path):
     """
     file_fd = os.open(file_path, os.O_RDONLY)
     file_stat = os.fstat(file_fd)
-    os.fsync(file_fd)
-    os.close(file_fd)
+    try:
+        os.fsync(file_fd)
+        return file_stat
+    except OSError as e:
+        # On some filesystem doing a fsync on a O_RDONLY fd
+        # raises an EACCES error. In that case we need to try again after
+        # reopening as O_RDWR.
+        if e.errno != errno.EACCES:
+            raise
+    finally:
+        os.close(file_fd)
+
+    file_fd = os.open(file_path, os.O_RDWR)
+    try:
+        os.fsync(file_fd)
+    finally:
+        os.close(file_fd)
+
     return file_stat
 
 
@@ -537,3 +569,46 @@ def check_positive(value):
         raise ArgumentTypeError("'%s' is not a valid positive integer" %
                                 value)
     return int_value
+
+
+def check_size(value):
+    """
+    Check user input for a human readable size
+
+    :param value: str containing the value to check
+    """
+    if value is None:
+        return None
+    # Ignore cases
+    value = value.upper()
+    try:
+        # If value ends with `B` we try to parse the multiplier,
+        # otherwise it is a plain integer
+        if value[-1] == 'B':
+            # By default we use base=1024, if the value ends with `iB`
+            # it is a SI value and we use base=1000
+            if value[-2] == 'I':
+                base = 1000
+                idx = 3
+            else:
+                base = 1024
+                idx = 2
+            multiplier = base
+            # Parse the multiplicative prefix
+            for prefix in 'KMGTPEZY':
+                if value[-idx] == prefix:
+                    int_value = int(float(value[:-idx]) * multiplier)
+                    break
+                multiplier *= base
+            else:
+                # If we do not find the prefix, remove the unit
+                # and try to parse the remainder as an integer
+                # (e.g. '1234B')
+                int_value = int(value[:-idx + 1])
+        else:
+            int_value = int(value)
+    except ValueError:
+        raise ArgumentTypeError("'%s' is not a valid size string" % value)
+    if int_value is None or int_value < 1:
+        raise ArgumentTypeError("'%s' is not a valid size string" % value)
+    return int_value
